package function.builtin;

import java.util.HashMap;

import function.*;
import sexpression.*;

public class DEFUN extends LispFunction {

    private ArgumentValidator argumentValidator;

    public DEFUN() {
        this.argumentValidator = new ArgumentValidator("DEFUN");
        this.argumentValidator.setMinimumNumberOfArguments(3);
        this.argumentValidator.setFirstArgumentExpectedType(Symbol.class);
    }

    public SExpression call(Cons argumentList) {
        argumentValidator.validate(argumentList);

        Cons cdr = (Cons) argumentList.getCdr();
        SExpression name = argumentList.getCar(); // name of the function
        SExpression cadr = cdr.getCar();

        // make sure the list of arguments (lambda list) is a proper list
        if (!cadr.listp()) {
            throw new RuntimeException("DEFUN: " + cadr + " is not a list");
        } else if (EVAL.isDotted((Cons) cadr)) {
            throw new RuntimeException("DEFUN: " + cadr + " is not a proper list");
        }

        Cons lambdaList = (Cons) cadr; // lambda list of the function

        // list of S-expressions making up the body of the function
        Cons body = (Cons) cdr.getCdr();

        HashMap<String, LispFunction> functionTable = EVAL.getFunctionTable();

        // give a warning if this function has already been defined
        if (functionTable.containsKey(name.toString())) {
            System.out.println("WARNING: redefining function " + name.toString());
        }

        // place the function in the function table
        functionTable.put(name.toString(), new UserDefinedFunction(name.toString(), lambdaList, body));

        return name;
    }

    public boolean evaluateArguments() {
        return false;
    }

}
